(*
 JungTaek Kim, The LET Project, KAIST

 Copyright(c) 2000 Research On Program Analysis System
 National Creative Research Initiative Center
 Korea Advanced Institute of Science & Technology
 http://ropas.kaist.ac.kr

 All rights reserved. This file is distributed under the terms of
 an Open Source License.
*)

open Location
open Format
open Lexing

(*
let parse_file filename =
  let ic = open_in_bin filename in
  let ast =
    try
      Location.input_name := filename;
      Parse.parse (Lexing.from_channel ic)
    with x -> close_in ic; raise x;
  in
    Printast.print_topdec ast;
    close_in ic
*)

(* Reading function for interactive use *)

let first_line = ref true
let got_eof = ref false

let refill_lexbuf buffer len =
  if !got_eof then (got_eof := false; 0) else begin
    output_string stdout (if !first_line then "# " else "  "); flush stdout;
    first_line := false;
    let i = ref 0 in
    try
      while !i < len && (let c = input_char stdin in buffer.[!i] <- c; c<>'\n')
      do incr i done;
      !i + 1
    with End_of_file ->
      Location.echo_eof ();
      if !i > 0
      then (got_eof := true; !i)
      else 0
  end

(* Discard everything already in a lexer buffer *)

let empty_lexbuf lb =
  let l = String.length lb.lex_buffer in
   lb.lex_abs_pos <- (-l);
   lb.lex_curr_pos <- l

let loop () = 
  print_string "           N Language Test Parser ";
  print_newline();
  let lb = from_function refill_lexbuf in
  Location.input_name := "";
  Location.input_lexbuf := Some lb;
  Sys.catch_break true;
  while true do
    try
      empty_lexbuf lb;
      Location.reset();
      first_line := true;
      let ast = Parse.interactive_parse lb in
        (String_ast.Ast.print_topdec ast; print_newline())
    with
      End_of_file -> exit 0
    | Sys.Break ->
        print_string "Interrupted."; print_newline();
    | Lexer.Error(err, start, stop) ->
        Location.print {loc_start = start; loc_end = stop; loc_ghost=false};
        Lexer.report_error err; print_newline();
    | Syntaxerr.Error err ->
        Syntaxerr.report_error err; print_newline();
    | x -> printf "@.\n"; raise x
  done

let optionlist =
  [
   ("-hangul", Arg.Unit 
   (fun () -> 
     (match !(Syntaxerr.errmsg_type) with
       Syntaxerr.EngGentle -> Syntaxerr.errmsg_type := Syntaxerr.HanGentle
     | _ -> ())),
   "Turn on Hangul syntax error messages.");
   ("-tough", Arg.Unit
   (fun () -> 
     (match !(Syntaxerr.errmsg_type) with
       Syntaxerr.HanGentle -> Syntaxerr.errmsg_type := Syntaxerr.HanTough
     | _ -> ())),
   "Turn on Tough syntax error messages.")
   ]

let usage = "Usage: nml\nOptions are:"

let main () =
  Format.set_formatter_out_channel stderr;
  Arg.parse optionlist (fun _ -> ()) usage;
  loop();
  printf "@."

let _ = Printexc.catch main () 
