(*
 JungTaek Kim, The LET Project, KAIST

 Copyright(c) 2000 Research On Program Analysis System
 National Creative Research Initiative Center
 Korea Advanced Institute of Science & Technology
 http://ropas.kaist.ac.kr

 All rights reserved. This file is distributed under the terms of
 an Open Source License.
*)

open Location

let batch_parse lexbuf = 
  try
    let ast = Parser.batch_parse Lexer.token lexbuf in
    Postparse.check_error ast;
    Parsing.clear_parser();
    ast
  with
      Lexer.Error(Lexer.Unterminated_comment, _, _) as err -> raise err
    | Lexer.Error(Lexer.Unterminated_string, _, _) as err -> raise err
    | Lexer.Error(_, _, _) as err -> raise err
    | Syntaxerr.Error _ as err -> raise err
    | Parsing.Parse_error | Syntaxerr.Escape_error ->
        let loc = { loc_start = Lexing.lexeme_start lexbuf;
                    loc_end = Lexing.lexeme_end lexbuf;
		    loc_ghost = false } in
        raise(Syntaxerr.Error(Syntaxerr.Other(loc,Syntaxerr.UnknownError)))

let interactive_parse lexbuf = 
  try
    let ast = Parser.interactive_parse Lexer.token lexbuf in
    Postparse.check_error ast;
    Parsing.clear_parser();
    ast
  with
      Lexer.Error(Lexer.Unterminated_comment, _, _) as err -> raise err
    | Lexer.Error(Lexer.Unterminated_string, _, _) as err -> raise err
    | Lexer.Error(_, _, _) as err -> raise err
    | Syntaxerr.Error _ as err -> raise err
    | Parsing.Parse_error | Syntaxerr.Escape_error ->
        let loc = { loc_start = Lexing.lexeme_start lexbuf;
                    loc_end = Lexing.lexeme_end lexbuf;
		    loc_ghost = false } in
        raise(Syntaxerr.Error(Syntaxerr.Other(loc,Syntaxerr.UnknownError)))

